@@ -18,6 +18,8 @@ import sys
 import os
 import platform
 import traceback
+import logging
+import yaml
 from datetime import datetime
 from pathlib import Path, PurePosixPath
 
@@ -27,7 +29,7 @@ from djsciops import __version__ as axon_version
 from djsciops import axon as djsciops_axon
 from djsciops import authentication as djsciops_authentication
 from djsciops import settings as djsciops_settings
-from djsciops import utils as djsciops_utils
+from djsciops.log import log, log_format
 import djsciops
 
 from djsciops.axon_gui.modules import *
@@ -85,6 +87,13 @@ class Worker(QRunnable):
         self.args = args
         self.kwargs = kwargs
         self.signals = WorkerSignals()
+        self.signals.error.connect(self.log_exception)
+
+    def log_exception(self, exc):
+        if log.getEffectiveLevel() == 10:
+            log.debug(f"Uncaught exception: {exc[2]}")
+        else:
+            log.error(f"Uncaught exception: {exc[1]}")
 
     @Slot()
     def run(self):
@@ -116,6 +125,40 @@ class MainWindow(QMainWindow):
         global widgets
         widgets = self.ui
 
+        # DEFINE LOG HANDLER
+        # ///////////////////////////////////////////////////////////////
+        class Main(QObject):
+            log = Signal(str)
+
+        class GuiLogHandler(logging.Handler):
+            def __init__(self, emitter):
+                super().__init__()
+                self._emitter = emitter
+                self.widget = widgets.upload_status
+                self._emitter.log.connect(self.widget.append)
+                # self.widget.verticalScrollBar().setValue(
+                #     self.widget.verticalScrollBar().maximum()
+                # )
+                # self.widget.ensureCursorVisible()
+
+            @property
+            def emitter(self):
+                return self._emitter
+
+            def emit(self, record):
+                msg = self.format(record)
+                self.emitter.log.emit(msg)
+
+        gui_handler = GuiLogHandler(Main())
+        gui_handler.setFormatter(
+            logging.Formatter("[%(asctime)s][%(levelname)s]: %(message)s", "%H:%M:%S")
+        )
+        log.addHandler(gui_handler)
+
+        # SET VERSION
+        # ///////////////////////////////////////////////////////////////
+        widgets.version.setText(f"v{axon_version}")
+
         # USE CUSTOM TITLE BAR | USE AS "False" FOR MAC OR LINUX
         # ///////////////////////////////////////////////////////////////
         Settings.ENABLE_CUSTOM_TITLE_BAR = True
@@ -154,18 +197,7 @@ class MainWindow(QMainWindow):
 
         # SET CONFIG
         # ///////////////////////////////////////////////////////////////
-        try:
-            self.config = djsciops_settings.get_config(stdin_enabled=False)
-        except FileNotFoundError:
-            self.config = dict(
-                version=axon_version,
-                aws=dict(account_id=""),
-                s3=dict(role="", bucket=""),
-                djauth=dict(client_id=""),
-            )
-            widgets.upload_status.setText(
-                "-> Set up your djsciops config to begin uploading"
-            )
+        self.config = djsciops_settings.get_config(stdin_enabled=False)     
 
         # BUTTONS CLICK
         # ///////////////////////////////////////////////////////////////
@@ -257,12 +289,9 @@ class MainWindow(QMainWindow):
                 str(QFileDialog.getExistingDirectory(self, "Select Directory"))
             )
             if widgets.local_dir.text():
-                try:
-                    UIFunctions.refresh_tree(
-                        widgets.local_dir_tree, widgets.local_dir.text(), "path"
-                    )
-                except Exception as e:
-                    djsciops_utils.log(f"Failed to load directory: {e}", gui=widgets)
+                UIFunctions.refresh_tree(
+                    widgets.local_dir_tree, widgets.local_dir.text(), "path"
+                )
                 if widgets.s3_dir.text():
                     self.load_bucket_preview()
             else:
@@ -272,24 +301,26 @@ class MainWindow(QMainWindow):
         # UPLOAD FILES
         elif btnName == "btn_upload_files":
             if not widgets.local_dir.text() or not widgets.s3_dir.text():
-                djsciops_utils.log(
-                    "Select a source and destination to begin uploading", gui=widgets
-                )
+                log.warning("Select a source and destination to begin uploading")
             elif (
                 not widgets.aws_acc_id.text()
                 or not widgets.dj_acc_client_id.text()
                 or not widgets.s3_role.text()
                 or not widgets.s3_bucket.text()
             ):
-                djsciops_utils.log(
-                    "Set up your djsciops config to begin uploading",
-                    gui=widgets,
-                )
+                log.warning("Set up your djsciops config to begin uploading")
             elif self.check_auth_token():
-                worker = Worker(self.upload_files)
-                worker.signals.finished.connect(
-                    lambda: widgets.btn_upload_files.setEnabled(True)
+                widgets.btn_upload_files.setEnabled(False)
+                checked_items, dest = self.map_checked_items(widgets.local_dir_tree)
+                worker = Worker(
+                    djsciops_axon.upload_files,
+                    session=self.session,
+                    s3_bucket=self.config["s3"]["bucket"],
+                    source=checked_items,
+                    destination=dest,
                 )
+                worker.signals.result.connect(self.upload_result)
+                worker.signals.finished.connect(self.upload_finished)
                 self.threadpool.start(worker)
 
         # SAVE CONFIG
@@ -298,65 +329,52 @@ class MainWindow(QMainWindow):
             self.config["djauth"]["client_id"] = widgets.dj_acc_client_id.text()
             self.config["s3"]["role"] = widgets.s3_role.text()
             self.config["s3"]["bucket"] = widgets.s3_bucket.text()
-            try:
-                config_directory = djsciops_settings.appdirs.user_data_dir(
-                    appauthor="datajoint", appname="djsciops"
-                )
-                os.makedirs(config_directory, exist_ok=True)
-                with open(
-                    djsciops_settings.pathlib.Path(config_directory, "config.yaml"), "w"
-                ) as f:
-                    f.write(str(self.config))
-                djsciops_utils.log("Config has been saved", gui=widgets)
-            except Exception as e:
-                djsciops_utils.log(f"Failed to save config: {e}", gui=widgets)
+            config_directory = djsciops_settings.appdirs.user_data_dir(
+                appauthor="datajoint", appname="djsciops"
+            )
+            djsciops_settings.save_config(yaml.dump(self.config), config_directory)
+            log.info("Config has been saved")
 
         # PRINT BTN NAME
         print(f'Button "{btnName}" pressed!')
 
-    # WORKER FUNCTIONS
+    # WORKER SIGNAL FUNCTIONS
     # ///////////////////////////////////////////////////////////////
-    def upload_files(self):
-        try:
-            widgets.btn_upload_files.setEnabled(False)
-            checked_items, dest = self.map_checked_items(widgets.local_dir_tree)
-            djsciops_axon.upload_files(
-                session=self.session,
-                s3_bucket=self.config["s3"]["bucket"],
-                source=checked_items,
-                destination=dest,
-                gui=widgets,
-            )
-            djsciops_utils.log("Upload complete", gui=widgets)
-        except Exception as e:
-            djsciops_utils.log(f"Upload failed: {e}", gui=widgets)
+    def upload_finished(self):
+        widgets.btn_upload_files.setEnabled(True)
+
+    def upload_result(self, r):
+        log.info("Upload complete")
+
+    def auth_finished(self):
+        widgets.refreshTokenBtn.setEnabled(True)
+
+    def auth_result(self, s):
+        self.session = s
+        time = datetime.fromtimestamp(self.session.jwt["exp"]).strftime("%H:%M %p")
+        widgets.expTime.setText(f"Token Expiration Time: {time}")
+
+    # UTIL FUNCTIONS
+    # ///////////////////////////////////////////////////////////////
+    def generate_auth_token(self):
+        widgets.refreshTokenBtn.setEnabled(False)
+        worker = Worker(
+            djsciops_authentication.Session,
+            aws_account_id=self.config["aws"]["account_id"],
+            s3_role=self.config["s3"]["role"],
+            auth_client_id=self.config["djauth"]["client_id"],
+        )
+        worker.signals.result.connect(self.auth_result)
+        worker.signals.finished.connect(self.auth_finished)
+        self.threadpool.start(worker)
 
     def load_bucket_dir(self):
         if self.check_auth_token():
             UIFunctions.update_bucket_tree(self)
 
-    def generate_auth_token(self):
-        widgets.refreshTokenBtn.setEnabled(False)
-        worker = Worker(
-            lambda: djsciops_authentication.Session(
-                aws_account_id=self.config["aws"]["account_id"],
-                s3_role=self.config["s3"]["role"],
-                auth_client_id=self.config["djauth"]["client_id"],
-                gui=widgets,
-            )
-        )
-        worker.signals.finished.connect(self.finish_auth)
-        worker.signals.result.connect(self.set_session)
-        self.threadpool.start(worker)
-
-    # UTIL FUNCTIONS
-    # ///////////////////////////////////////////////////////////////
     def load_bucket_preview(self):
         if widgets.local_dir.text() and widgets.s3_dir.text():
-            try:
-                UIFunctions.update_bucket_preview(self)
-            except Exception as e:
-                djsciops_utils.log(f"Failed to load bucket preview: {e}", gui=widgets)
+            UIFunctions.update_bucket_preview(self)
 
     def map_checked_items(self, tree):
         items = []
@@ -409,37 +427,24 @@ class MainWindow(QMainWindow):
             widgets.themeSwitchBtn.setIcon(QIcon(":/icons/images/icons/cil-moon.png"))
 
     def check_auth_token(self):
-        djsciops_utils.log("authentication", message_type="header", gui=widgets)
         if not self.session:
-            djsciops_utils.log(
-                "Generate a new authentication token to begin using Axon", gui=widgets
-            )
+            log.warning("Generate a new authentication token to begin using Axon")
             return False
         else:
             time_to_live = (
                 (self.session.jwt["exp"] - datetime.now().timestamp()) / 60 / 60
             )
             if time_to_live <= 0:
-                djsciops_utils.log(
-                    "Authentication token has expired. Please generate a new token",
-                    gui=widgets,
+                log.warning(
+                    "Authentication token has expired. Please generate a new token"
                 )
                 return False
             else:
-                djsciops_utils.log(
-                    f"Using valid authentication token with a life of {'{:.2f}'.format(time_to_live)} [HR]",
-                    gui=widgets,
+                log.info(
+                    f"Using valid authentication token with a life of {'{:.2f}'.format(time_to_live)} [HR]"
                 )
                 return True
 
-    def finish_auth(self):
-        widgets.refreshTokenBtn.setEnabled(True)
-        time = datetime.fromtimestamp(self.session.jwt["exp"]).strftime("%H:%M %p")
-        widgets.expTime.setText(f"Token Expiration Time: {time}")
-
-    def set_session(self, s):
-        self.session = s
-
     # RESIZE EVENTS
     # ///////////////////////////////////////////////////////////////
     def resizeEvent(self, event):
