@@ -44,6 +44,24 @@ def djsciops(args: list = None):
         help="Target directory in object store.",
     )
 
+    axon_upload_optional_named = axon_upload.add_argument_group("optional named arguments")
+    axon_upload_optional_named.add_argument(
+        "-p", "--permit_regex",
+        type=str,
+        default=djsciops_axon._PERMIT_REGEX_DEFAULT,
+        required=False,
+        dest='permit_regex',
+        help="Regular expression pattern to permit file(s) for uploading (default - all files permitted).",
+    )
+    axon_upload_optional_named.add_argument(
+        "-i", "--ignore_regex",
+        type=str,
+        default=djsciops_axon._IGNORE_REGEX_DEFAULT,
+        required=False,
+        dest='ignore_regex',
+        help="Regular expression pattern to ignore file(s) for uploading (default - no files ignored).",
+    )
+
     axon_download = axon_subcommand.add_parser(
         "download", description="Download objects from object store."
     )
@@ -61,6 +79,24 @@ def djsciops(args: list = None):
         help="Target directory on client.",
     )
 
+    axon_download_optional_named = axon_download.add_argument_group("optional named arguments")
+    axon_download_optional_named.add_argument(
+        "-p", "--permit_regex",
+        type=str,
+        default=djsciops_axon._PERMIT_REGEX_DEFAULT,
+        required=False,
+        dest='permit_regex',
+        help="Regular expression pattern to permit file(s) for downloading (default - all files permitted).",
+    )
+    axon_download_optional_named.add_argument(
+        "-i", "--ignore_regex",
+        type=str,
+        default=djsciops_axon._IGNORE_REGEX_DEFAULT,
+        required=False,
+        dest='ignore_regex',
+        help="Regular expression pattern to ignore file(s) for downloading (default - no files ignored).",
+    )
+
     config = command.add_parser("config", description="View or modify djsciops config")
 
     config_optional_named = config.add_argument_group("optional named arguments")
@@ -84,6 +120,8 @@ def djsciops(args: list = None):
                 s3_bucket=config["s3"]["bucket"],
                 source=kwargs["source"],
                 destination=kwargs["destination"],
+                permit_regex=kwargs["permit_regex"],
+                ignore_regex=kwargs["ignore_regex"]
             )
         elif kwargs["subcommand"] == "download":
             djsciops_axon.download_files(
@@ -95,6 +133,8 @@ def djsciops(args: list = None):
                 s3_bucket=config["s3"]["bucket"],
                 source=kwargs["source"],
                 destination=kwargs["destination"],
+                permit_regex=kwargs["permit_regex"],
+                ignore_regex=kwargs["ignore_regex"]
             )
     elif kwargs["command"] == "config":
 
